-- Create table
CREATE TABLE IF NOT EXISTS `{table}`
(
    `player_uuid` CHAR(36) CHARACTER SET ascii COLLATE ascii_bin    NOT NULL,
    `id` VARCHAR(50) CHARACTER SET ascii COLLATE ascii_bin NOT NULL,
    `value` VARCHAR(8192) CHARACTER SET utf8 COLLATE utf8_bin NOT NULL,
    UNIQUE `player_uuid_index` (`player_uuid`, `id`) USING BTREE
);

-- Increase value size
ALTER TABLE `{table}`
    MODIFY COLUMN `value` VARCHAR(8192) CHARACTER SET utf8 COLLATE utf8_bin NOT NULL;

-- Create player names table
-- We can't use the properties table as we need to index the name (for which the property column is too large)
CREATE TABLE IF NOT EXISTS `{table_names}`
(
    `player_uuid` CHAR(36) CHARACTER SET ascii COLLATE ascii_bin NOT NULL,
    `username` VARCHAR(16) CHARACTER SET ascii COLLATE ascii_bin NOT NULL,
    UNIQUE (`username`) USING BTREE
);

-- Create stored procedure used to loop through passed properties
-- After testing, this is the most efficient way to update player properties in MySQL
-- Transactions and batches were 2x slower in my testing
DROP procedure IF EXISTS `mbedwars_player_properties_update`;

CREATE PROCEDURE `mbedwars_player_properties_update`(_player_uuid CHAR(36) CHARACTER SET ascii COLLATE ascii_bin,
                                                     _raw_properties MEDIUMTEXT CHARACTER SET ascii COLLATE ascii_bin)
proced: BEGIN
    -- init everything
    DECLARE _end_index INT;
    DECLARE _id TEXT DEFAULT NULL;
    DECLARE _value TEXT CHARACTER SET utf8 COLLATE utf8_bin DEFAULT NULL;

    -- exit the proc if the list is empty
    if LENGTH(_raw_properties) = 0 then
        DELETE FROM `{table}`
            WHERE `player_uuid` = _player_uuid;
        LEAVE proced;
    end if;

    -- ok, we want some to be added
    DROP TABLE IF EXISTS _removal_properties;
    CREATE TEMPORARY TABLE _removal_properties
        (`id` VARCHAR(50) CHARACTER SET ascii COLLATE ascii_bin NOT NULL)
        ENGINE=MEMORY;
    INSERT INTO _removal_properties
        SELECT `id`
        FROM `{table}`
        WHERE `player_uuid` = _player_uuid;

    -- ### first go through all properties
    iterator: LOOP
        -- capture the next value from the list
        SET _end_index = POSITION(',' IN _raw_properties);

        if _end_index = 0 then
            SET _end_index = LENGTH(_raw_properties);
        else
            SET _end_index = _end_index - 1;
        end if;

        -- parse the entry
        SET _id = SUBSTRING_INDEX(_raw_properties, '!', 1);
        SET _value = FROM_BASE64(SUBSTR(_raw_properties, LENGTH(_id)+2, _end_index-LENGTH(_id)-1));

        -- update table
        REPLACE
            INTO `{table}` (`player_uuid`, `id`, `value`)
            VALUES (_player_uuid, _id, _value);

        -- tell memory table that we don't want to remove this entry later
        DELETE
            FROM _removal_properties
            WHERE `id` = _id;

        -- update username
        if _id = 'bedwars:username' then
            REPLACE
                INTO `{table_names}` (`player_uuid`, `username`)
                VALUES (_player_uuid, _value);
        end if;

        -- we are done!
        if _end_index = LENGTH(_raw_properties) then
            LEAVE iterator;
        end if;

        -- remove the part of the string we just processed
        SET _raw_properties = SUBSTR(_raw_properties, _end_index+2);
    END LOOP;

    -- ### remove left properties
    DELETE
    FROM `{table}`
    WHERE `player_uuid` = _player_uuid
      AND `id` IN (SELECT * FROM _removal_properties);
END;